<?php

namespace FiloBlu\Storelocator\Model;

use Magento\Framework\Api\AttributeValueFactory;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Rma\Api\RmaAttributesManagementInterface;
use Magento\Rma\Model\Rma\EntityAttributesLoader;

class Rma extends \Magento\Rma\Model\Rma
{
    /**
     * @var \FiloBlu\Storelocator\Helper\Data
     */
    protected $_helper;
    /**
     * @var \Magento\Rma\Helper\Data
     */
    protected $_rmaData;
    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;
    /**
     * @var \Magento\Framework\Translate\Inline\StateInterface
     */
    protected $_inlineTranslation;
    /**
     * @var StoresFactory
     */
    protected $_storesFactory;
    /**
     * @var \Magento\Payment\Helper\Data
     */
    protected $_paymentHelper;
    /**
     * @var \Magento\Framework\Mail\Template\TransportBuilder
     */
    protected $_transportBuilder;
    /**
     * @var \Magento\Sales\Model\Order\Address\Renderer
     */
    protected $_addressRenderer;
    /**
     * @var RequestInterface
     */
    protected $_request;

    public function __construct(
        \Magento\Framework\Model\Context $context ,
        \Magento\Framework\Registry $registry ,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory ,
        AttributeValueFactory $customAttributeFactory ,
        \Magento\Rma\Helper\Data $rmaData ,
        \Magento\Framework\Session\Generic $session ,
        \Magento\Store\Model\StoreManagerInterface $storeManager ,
        \Magento\Eav\Model\Config $eavConfig ,
        \Magento\Rma\Model\ItemFactory $rmaItemFactory ,
        \Magento\Rma\Model\Item\Attribute\Source\StatusFactory $attrSourceFactory ,
        \Magento\Rma\Model\GridFactory $rmaGridFactory ,
        \Magento\Rma\Model\Rma\Source\StatusFactory $statusFactory ,
        \Magento\Rma\Model\ResourceModel\ItemFactory $itemFactory ,
        \Magento\Rma\Model\ResourceModel\Item\CollectionFactory $itemsFactory ,
        \Magento\Rma\Model\ResourceModel\Shipping\CollectionFactory $rmaShippingFactory ,
        \Magento\Quote\Model\QuoteFactory $quoteFactory ,
        \Magento\Quote\Model\Quote\Address\RateFactory $quoteRateFactory ,
        \Magento\Quote\Model\Quote\ItemFactory $quoteItemFactory ,
        \Magento\Sales\Model\OrderFactory $orderFactory ,
        \Magento\Sales\Model\ResourceModel\Order\Item\CollectionFactory $ordersFactory ,
        \Magento\Quote\Model\Quote\Address\RateRequestFactory $rateRequestFactory ,
        \Magento\Shipping\Model\ShippingFactory $shippingFactory ,
        \Magento\Framework\Escaper $escaper ,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate ,
        \Magento\Framework\Message\ManagerInterface $messageManager ,
        \Magento\Rma\Api\RmaAttributesManagementInterface $metadataService ,

        \FiloBlu\Storelocator\Helper\Data $helper,
        \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation,
        \FiloBlu\Storelocator\Model\StoresFactory $storesFactory,
        \Magento\Payment\Helper\Data $paymentHelper,
        \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder,
        \Magento\Sales\Model\Order\Address\Renderer $addressRenderer,

        RequestInterface $request,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null ,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null ,
        array $data = [] ,
        Json $serializer = null ,
        EntityAttributesLoader $attributesLoader = null
    )
    {
        parent::__construct($context , $registry , $extensionFactory , $customAttributeFactory , $rmaData , $session , $storeManager , $eavConfig , $rmaItemFactory , $attrSourceFactory , $rmaGridFactory , $statusFactory , $itemFactory , $itemsFactory , $rmaShippingFactory , $quoteFactory , $quoteRateFactory , $quoteItemFactory , $orderFactory , $ordersFactory , $rateRequestFactory , $shippingFactory , $escaper , $localeDate , $messageManager , $metadataService , $resource , $resourceCollection , $data , $serializer , $attributesLoader);
        $this->_helper = $helper;
        $this->_rmaData = $rmaData;
        $this->_storeManager = $storeManager;
        $this->_inlineTranslation = $inlineTranslation;
        $this->_storesFactory = $storesFactory;
        $this->_paymentHelper = $paymentHelper;
        $this->_transportBuilder = $transportBuilder;
        $this->_addressRenderer = $addressRenderer;
        $this->_request = $request;
    }

    public function saveRma($data)
    {
        if ($this->_helper->getPickupInStoreEnabled()) {
            $x = null;
        }


        // TODO: move errors adding to controller
        $errors = 0;
        $this->messageManager->getMessages(true);
        if ($this->getCustomerCustomEmail()) {
            $validateEmail = $this->_validateEmail($this->getCustomerCustomEmail());
            if (is_array($validateEmail)) {
                foreach ($validateEmail as $error) {
                    $this->messageManager->addError($error);
                }
                $this->_session->setRmaFormData($data);
                $errors = 1;
            }
        }

        $itemModels = $this->_createItemsCollection($data);
        if (!$itemModels || $errors) {
            return false;
        }

        if(isset($data['rma_returntostore_store_id']) && $data['rma_returntostore_store_id'] > 0) {
            $this->setData('return_to_store_id', $data['rma_returntostore_store_id']);
            $this->setData('return_to_store_email', $data['rma_returntostore_email']);
            $this->setData('return_to_store_hash', (md5(time())));
        }


        try {
            $this->save();
            $this->sendEmails($this);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
        return $this;

        //return parent::saveRma($data); // TODO: Change the autogenerated stub
    }

    public function sendEmails($rma) {

        $returnInStoreEnabled = $this->_helper->getReturnToStoreEnabled();
        $useStandardMailFlow = $this->_helper->getUseStandardRmaMailFlow();
        $returnToStoreId = (int)$rma->getReturnToStoreId();

        if (!$returnInStoreEnabled) {
            return $rma;
        }

        if ($returnToStoreId) {
            if ($rma->getStatus() == 'pending') {
                try {
                    if ($rma->getData('return_to_store_received_date') == null) {
                        $this->_helper->sendRmaMail($rma);
                    }
                } catch (\Exception $ex) {

                }
            }
        } else {
            try {
                if ($rma->getData('return_to_store_received_date') == null) {
                    $this->_helper->sendRmaMail($rma);
                }
            } catch (\Exception $ex) {

            }
        }

        /*
        if (!$useStandardMailFlow && $returnToStoreId) {
            if ($rma->getStatus() == 'pending') {
                try {
                    if ($rma->getData('return_to_store_received_date') == null) {
                        $this->sendMail($rma);
                    }
                } catch (\Exception $ex) {

                }
            }
        } else if (!$useStandardMailFlow && !(int)$rma->getReturnToStoreId()) {
            try {
                if ($rma->getData('return_to_store_received_date') == null) {
                    $this->sendMail($rma);
                }
            } catch (\Exception $ex) {

            }
        }*/

    }

    protected function sendMail($rma)
    {
        if (!$this->_helper->getConfig("carriers/returninstore/pickrma_email_to_store_enabled")) {
            return;
        }

        $storeId = (int)$rma->getReturnToStoreId();
        $shop = $this->getStoreDetails($storeId);

        $mailReceiver = $rma->getReturnToStoreEmail() ? $rma->getReturnToStoreEmail() : $this->_helper->getCustomercareEmail();

        $this->_inlineTranslation->suspend();

        $mailTo = $this->_helper->getConfig("carriers/returninstore/pickrma_sender_email_to_store_identity");

        $template = $this->_helper->getConfig("carriers/returninstore/pickrma_email_to_store_template");

        $data = [];

        if (!$this->_helper->getUseStandardRmaMailFlow() && $storeId) {
            if ($rma->getStatus() == 'pending') {
                $mailReceiver = $this->_helper->getCustomercareEmail();
                $template = $this->_helper->getReturnInStoreEmailTemplate();
            }

            if ($rma->getStatus() == 'authorized') {

                $data = $this->_helper->getStoreDetails($storeId);

                if ($this->_helper->getUseTestEmailRecipients()) {
                    if ($this->_helper->getStoreTestEmail()) {
                        $mailReceiver = $this->_helper->getStoreTestEmail();
                    }
                }
            }
        }

        try {
            $returnAddress = $this->_rmaData->getReturnAddress('html', $data, $storeId);
            $order = $rma->getOrder();

            $transport = $this->_transportBuilder
                ->setTemplateIdentifier($template)
                ->setTemplateOptions(['area' => \Magento\Framework\App\Area::AREA_FRONTEND, 'store' => $this->_storeManager->getStore()->getStoreId()])
                ->setTemplateVars(
                    [
                        'rma' => $rma,
                        'order' => $rma->getOrder(),
                        'shop' => $shop,
                        'store' => $rma->getOrder()->getStore(),
                        'billing' => $order->getBillingAddress(),
                        'payment_html' => $this->_paymentHelper->getInfoBlockHtml(
                            $order->getPayment(),
                            $this->_storeManager->getStore()->getId()
                        ),
                        'return_address' => $returnAddress,
                        'item_collection' => $rma->getItemsForDisplay(),
                        'formattedShippingAddress' => $this->_addressRenderer->format(
                            $order->getShippingAddress(),
                            'html'
                        ),
                        'formattedBillingAddress' => $this->_addressRenderer->format(
                            $order->getBillingAddress(),
                            'html'
                        ),
                        'sendRejectionEmail' => true
                    ]
                )
                ->setFrom($mailTo)
                ->addTo($mailReceiver)
                ->getTransport();
            $transport->sendMessage();

            $this->_inlineTranslation->resume();
        } catch (\Exception $e) {
            $this->_inlineTranslation->resume();
        }
    }

    protected function getStoreDetails($store_id)
    {
        if ((int)$store_id > 0) {
            $shop = $this->_storesFactory->create()->load($store_id);

            return $shop;
        }

        return false;
    }

}
